'''
Game manager for controlling in-game events.
'''

import sys
import pygame
from pygame.locals import *

from random import randint

from . import loader
from . import game_object
from . import scenario_mgmt

class GameManager():
    # class variables
    ORIGINAL_WIDTH = 320
    ORIGINAL_HEIGHT = 240

    SCREEN_WIDTH = ORIGINAL_WIDTH * 2
    SCREEN_HEIGHT = ORIGINAL_HEIGHT * 2
    SCREEN_RECT = (SCREEN_WIDTH, SCREEN_HEIGHT)

    LOADER = []

    def __init__(self):

        # for the workaround of pygame sound delay issue
        pygame.mixer.pre_init(44100, -16, 2, 512)

        pygame.init()
        self.expanded_screen = pygame.display.set_mode(GameManager.SCREEN_RECT)
        pygame.display.set_caption('Two Wolrds - A Game About Two Worlds')
        self.original_screen = pygame.Surface((GameManager.ORIGINAL_WIDTH, GameManager.ORIGINAL_HEIGHT))
        self.fps_clock = pygame.time.Clock()
        pygame.mouse.set_visible(0)

        # for bg brightness
        self.brightness_image = pygame.Surface(GameManager.SCREEN_RECT)
        self.brightness_image.set_colorkey((255, 255, 255))
        self.brightness_image = self.brightness_image.convert()
        self.brightness_image.fill((0, 0, 0))
        self.brightness = 136
        self.brightness_image.set_alpha(self.brightness)

        GameManager.LOADER = loader.Loader()

        self.initialize()

    def initialize(self):
        self.scenario = scenario_mgmt.ScenarioManager()
        self.player_cursor = game_object.Cursor()
        self.prompt =  game_object.Prompt()

        # test code
        self.bg_image = GameManager.LOADER.load_image("scene1.png")
        self.original_screen.fill((0, 0, 0))
        self.original_screen.blit(self.bg_image, (0,0))
        self.original_screen.blit(self.brightness_image, (0, 0))
        self.original_screen = self.original_screen.convert()

        # system variables
        self.full_screen = False
        self.fps = 60

        # default value
        self.default_life = 6
        self.life = self.default_life
        self.player_status = "alive"
        self.player_cursor.toggle_invincible() # start being invincible while prompting

        self.prompt_timing_cnt = 0
        self.prompt_timing_max = 2
        self.prompt.move(self.scenario.get_position())
        self.prompt.visible = 1

        self.game_phase = "prompting"
        self.game_end_flag = False

        self.char_block_list = []
        self.particle_list = []
        self.allsprites = []

    def title(self):
        return 0

    def ending(self):
        pass
   
    def game_over(self):
        pass

    def start(self):
        # play the main BGM
        pygame.mixer.music.set_volume(0.5)
        pygame.mixer.music.play(-1, 0) # loop position can't work for unknown reason

        self.player_cursor.visible = 1
        ret = self.run()

        pygame.mixer.music.stop()
        return ret

    def run(self):
        going = True
        while going:

            for event in pygame.event.get():
                if event.type == QUIT:
                    pygame.quit()
                    sys.exit()
                elif event.type == pygame.KEYDOWN:
                    keys = pygame.key.get_pressed()
                    if event.key == pygame.K_ESCAPE:
                        pygame.event.post(pygame.event.Event(pygame.QUIT))
                    elif keys[pygame.K_LALT] and keys[pygame.K_RETURN]:
                        if self.full_screen == False:
                            self.full_screen = True
                            pygame.display.set_mode(GameManager.SCREEN_RECT, FULLSCREEN)
                        else:
                            pygame.display.set_mode(GameManager.SCREEN_RECT)
                            self.full_screen = False
                elif event.type == pygame.MOUSEBUTTONDOWN:
                    if pygame.mouse.get_pressed()[0] == True:
                        if self.game_phase == "waiting for attack" and self.player_status == "alive":
                            GameManager.LOADER.sound_dict["choice"].play()
                            self.phase_start_attacking()
                        elif self.game_phase == "waiting for game over":
                            return "game over"
                        else:
                            GameManager.LOADER.sound_dict["cancel"].play()
                elif event.type == MOUSEMOTION:
                    self.player_cursor.move()

            if self.player_cursor.invincible == 0:
                # pixel perfect collision
                if pygame.sprite.spritecollide(self.player_cursor, self.char_block_list, False, pygame.sprite.collide_mask):
                    self.player_status = "dead"
                    GameManager.LOADER.sound_dict["damaged"].play()
                    self.player_cursor.toggle_invincible()
                    # spilling blood
                    max = randint(15, 22)
                    for i in range(0, max):
                        self.particle_list.append(game_object.BloodParticle(self.player_cursor.rect))
                    self.life -= 1
                    if self.life <= 0:
                        self.player_cursor.visible = 0
                        self.scenario.set_game_over()
                        self.prompt.move(self.scenario.get_position())
                        self.prompt.visible = 1
                        GameManager.LOADER.sound_dict["game over"].play()
                        self.game_phase = "game over"

            # invincible timer
            if self.player_status == "dead" and self.game_phase == "attacking":
                ret = self.player_cursor.invincible_timer()
                if ret == 1:
                    self.player_cursor.toggle_invincible()
                    self.player_status = "alive"

            if self.game_phase == "prompting":
                self.phase_prompting()
            elif self.game_phase == "waiting for attack":
                self.phase_waiting_for_attack()
            elif self.game_phase == "attacking":
                self.phase_attacking()
            elif self.game_phase == "game over":
                self.phase_game_over()
            elif self.game_phase == "waiting for game over":
                self.phase_waiting_for_game_over()
            elif self.game_phase == "ending":
                self.phase_ending()
            elif self.game_phase == "completed":
                return "completed"

            # blood particle action
            if len(self.particle_list) != 0:
                for par_obj in self.particle_list:
                    if par_obj.destroy_flag == 1:
                        self.particle_list.remove(par_obj)
                    else:
                        par_obj.move()
            
            self.original_screen.blit(self.bg_image, (0,0))
            self.original_screen.blit(self.brightness_image, (0, 0))
            self.allsprites = pygame.sprite.LayeredDirty(self.char_block_list, self.prompt, self.player_cursor, self.particle_list)
            self.allsprites.draw(self.original_screen)
            pygame.transform.scale2x(self.original_screen, self.expanded_screen)
            pygame.display.update()
            self.allsprites.update()
            self.fps_clock.tick(self.fps)

        return "aborted"

    def phase_prompting(self):
        self.prompt_timing_cnt = 0
        ret = 0
        text = self.scenario.read()
        if text == "end of block":
            self.game_phase = "waiting for attack"
            ret = self.scenario.seek()
        elif text == "LF" or text == "space":
            ret = self.scenario.seek()
            pos = self.scenario.get_position()
            self.prompt.move(pos)
        else:
            pos = self.scenario.get_position()
            self.prompt.move(pos)
            col = int(255-(self.default_life-self.life)*(240/self.default_life))
            if col < 0: col = 0
            self.char_block_list.append(game_object.CharBlock(text, pos, (255, col, col)))
            ret = self.scenario.seek()

        if ret == "end of scenario":
            self.prompt.visible = 0
            self.game_phase = "ending" 
            
    def phase_waiting_for_attack(self):
        self.prompt_timing_cnt += 1
        if self.prompt_timing_cnt > self.prompt_timing_max:
            self.prompt.blink_animation()
            self.promt_timing_cnt = 0

    def phase_start_attacking(self):
        self.prompt.visible = 0
        for obj in self.char_block_list:
            obj.set_vector(self.player_cursor.rect)

        self.player_cursor.toggle_invincible()
        self.game_phase = "attacking"

    def phase_attacking(self):
        # attack
        destroyed_cnt = 0
        for char_obj in self.char_block_list:
            if char_obj.destroy_flag == 1:
                destroyed_cnt += 1
            else:
                char_obj.move()

        if destroyed_cnt == len(self.char_block_list):
            # back to prompting
            self.char_block_list = []
            self.scenario.seek_next_head()
            self.prompt.initialize()
            self.prompt.move(self.scenario.get_position())
            self.prompt.visible = 1
            if self.player_status == "dead":
                self.player_status = "alive"
            else:
                self.player_cursor.toggle_invincible()
            self.game_phase = "prompting"

    def phase_game_over(self):
        self.prompt_timing_cnt = 0
        ret = 0
        text = self.scenario.read()
        if text == "LF" or text == "space":
            ret = self.scenario.seek()
            pos = self.scenario.get_position()
            self.prompt.move(pos)
        else:
            pos = self.scenario.get_position()
            self.prompt.move(pos)
            self.char_block_list.append(game_object.CharBlock(text, pos, (255, 0, 0)))
            ret = self.scenario.seek()

        if ret == "end of scenario":
            self.game_phase = "waiting for game over"

        for char_obj in self.char_block_list:
            char_obj.move()

    def phase_waiting_for_game_over(self):
        self.phase_waiting_for_attack()
        for char_obj in self.char_block_list:
            char_obj.move()

    def phase_ending(self):
        self.game_phase = "completed"
