'''
Game character behaviors.
'''

import pygame
from pygame.locals import *

import math
from random import randint

from . import game_mgmt

#Super class for game objects
class ActiveObject(pygame.sprite.DirtySprite):
    def __init__(self):
        pygame.sprite.DirtySprite.__init__(self)

        self.image = None
        self.rect = None

    def screen_out_check(self):
        if self.rect[0]+self.rect[2] < 0:
            return 1
        elif self.rect[0] > game_mgmt.GameManager.ORIGINAL_WIDTH:
            return 1
        elif self.rect[1]+self.rect[3] < 0:
            return 1
        elif self.rect[1] > game_mgmt.GameManager.ORIGINAL_HEIGHT:
            return 1
        else:
            return 0

    def update(self):
        self.dirty = 1

class Cursor(ActiveObject):
    def __init__(self):
        super(Cursor, self).__init__()

        self.normal_alpha = 180
        self.invincible_alpha = 100
        self.image = game_mgmt.GameManager.LOADER.load_image("normal_select.png", 0)
        self.image.set_alpha(self.normal_alpha)
        self.mask = pygame.mask.from_surface(self.image)  # for pixel perfect collision
        self.rect = self.image.get_rect()
        self.rect.topleft = (game_mgmt.GameManager.ORIGINAL_WIDTH/2, game_mgmt.GameManager.ORIGINAL_HEIGHT/2)

        self.invincible = 0
        self.invincible_time_max = 60
        self.invincible_time_cnt = 1

        self.initialize()

    def initialize(self):
        self.visible = 0

    def move(self):
        pos = pygame.mouse.get_pos()
        temp_rect = [pos[0]/2,  pos[1]/2]

        if temp_rect[0] + 12 >= game_mgmt.GameManager.ORIGINAL_WIDTH:
            temp_rect[0] = game_mgmt.GameManager.ORIGINAL_WIDTH - 12
        if temp_rect[1] + 20 >= game_mgmt.GameManager.ORIGINAL_HEIGHT:
            temp_rect[1] = game_mgmt.GameManager.ORIGINAL_HEIGHT - 20
        self.rect.topleft = (temp_rect[0],  temp_rect[1])

    def invincible_timer(self):
        self.invincible_time_cnt += 1
        if self.invincible_time_cnt >= self.invincible_time_max:
            return 1
        return 0

    def toggle_invincible(self):
        if self.invincible == 0:
            self.invincible = 1
            self.invincible_time_cnt = 1
            self.image.set_alpha(self.invincible_alpha)
        else:
            self.invincible = 0
            self.image.set_alpha(self.normal_alpha)

class CharBlock(ActiveObject):
    def __init__(self, alphabet, position, color):
        super(CharBlock, self).__init__()

        font = game_mgmt.GameManager.LOADER.get_font("main")
        self.image = pygame.Surface((16-4, 24+6)) # 28pt font size
        self.image.set_colorkey((0, 0, 0))
        self.image = self.image.convert()
        self.image.fill((0, 0, 0))
        text_color = color
        text = font.render(alphabet, False, text_color)  # no anti-aliasing
        self.image.blit(text, (0, 0))
        self.mask = pygame.mask.from_surface(self.image)  # for pixel perfect collision
        self.rect = self.image.get_rect()
        self.rect.topleft = position
        self.visible = 1

        #self.waiting_time = 0
        if alphabet in "abcdefg":
            self.speed = 5
        elif alphabet in "vwxyz":
            self.speed = 4
        elif alphabet in ".'":
            self.speed = 7
        elif alphabet in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
            self.speed = 9
        elif alphabet in "!?'":
            self.speed = 14
        else:
            self.speed = 6
        self.dx = 0
        self.dy = 0
        self.destroy_flag = 0

    def set_vector(self, target_pos):
        rad = math.atan2(self.rect[0] - target_pos[0], self.rect[1] - target_pos[1])
        angle = ((rad * (180.0 / math.pi)) + 90.0 + randint(0,20)) % 360.0
        self.dx = int( math.cos(angle * (math.pi / 180)) * self.speed )
        self.dy = int( math.sin(angle * (math.pi / 180)) * self.speed ) * -1

    def move(self):
        self.rect.topleft = (self.rect[0] + self.dx,  self.rect[1] + self.dy)
        if self.screen_out_check() == 1:
            self.destroy_flag = 1
        
class Prompt(ActiveObject):
    def __init__(self):
        super(Prompt, self).__init__()

        self.shape_full = pygame.Surface((16-4, 24+6)) # 28pt font size
        self.shape_full.set_colorkey((0, 0, 0))
        self.shape_full = self.shape_full.convert()
        self.shape_full.fill((255, 255, 255))
        self.rect = self.shape_full.get_rect()

        self.shape_half = pygame.Surface((16-4, 24+6)) # 28pt font size
        self.shape_half.set_colorkey((0, 0, 0))
        self.shape_half = self.shape_half.convert()
        pygame.draw.rect(self.shape_half, (255, 255, 255), (0, (24+6)-8, 16-4, 24+6))

        self.initialize()

    def initialize(self):
        self.image = self.shape_full
        self.rect.topleft = (0, 0)
        self.visible = 0

        self.blink_status = 0
        self.blink_cnt_max = 24
        self.blink_cnt = 0

    def move(self, position):
        self.rect.topleft = (position[0], position[1])
        
    def blink_reset(self):
        self.blink_cnt = 0
        self.image = self.shape_full

    def blink_animation(self):
        self.blink_cnt += 1
        if self.blink_cnt >= self.blink_cnt_max:
            self.blink_cnt = 0
            if self.blink_status == 0:
                self.blink_status = 1
                self.image = self.shape_half
            else:
                self.blink_status = 0
                self.image = self.shape_full

class BloodParticle(ActiveObject):
    def __init__(self, position):
        super(BloodParticle, self).__init__()

        size = (randint(2,8), randint(2,8))
        self.image = pygame.Surface(size)
        self.image = self.image.convert()
        self.image.fill((randint(128,255), 0, 0))
        self.image.set_alpha(100)
        self.rect = self.image.get_rect()
        self.rect.topleft = (position[0],  position[1])
        self.visible = 1

        self.gravity = 0.2
        self.destroy_flag = 0
        self.life_time = randint(5, 25)

        angle = randint(0,359)
        self.dx = int( math.cos(angle * (math.pi / 180)) * randint(1,4) )
        self.dy = int( math.sin(angle * (math.pi / 180)) * randint(2,6) ) * -1

    def move(self):
        self.rect.topleft = (self.rect[0] + self.dx,  self.rect[1] + self.dy)
        if self.screen_out_check() == 1:
            self.destroy_flag = 1

        self.life_time -= 1
        if self.life_time < 0:
            self.destroy_flag = 1

        self.dy += self.gravity
